/**
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.apache.geronimo.directory;

import java.io.File;
import java.net.MalformedURLException;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.directory.InitialDirContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.directory.server.configuration.MutableServerStartupConfiguration;
import org.apache.directory.server.core.configuration.ShutdownConfiguration;
import org.apache.directory.server.jndi.ServerContextFactory;
import org.apache.geronimo.gbean.GBeanInfo;
import org.apache.geronimo.gbean.GBeanInfoBuilder;
import org.apache.geronimo.gbean.GBeanLifecycle;
import org.apache.geronimo.system.serverinfo.ServerInfo;
import org.apache.xbean.spring.context.FileSystemXmlApplicationContext;
import org.springframework.context.ApplicationContext;

/**
 * Directory GBean
 *
 * @version $Rev: 563580 $ $Date: 2007-08-07 10:34:27 -0700 (Tue, 07 Aug 2007) $
 */
public class DirectoryGBean implements GBeanLifecycle {

    private static final Log log = LogFactory.getLog(DirectoryGBean.class);

    /**
     * Geronimo class loader
     */
    private ClassLoader classLoader;
    private static final String DEFAULT_WORKING_DIRECTORY = "var/directory";

    public DirectoryGBean(String workingDir,
                          String configFile,
                          String providerURL, ServerInfo serverInfo,
                          ClassLoader classLoader) throws NamingException, MalformedURLException {

        if (serverInfo == null) {
            throw new IllegalArgumentException("Must have a ServerInfo value in initParams.");
        }
        if (workingDir == null) {
            workingDir = DEFAULT_WORKING_DIRECTORY;
        }
        this.classLoader = classLoader;
        File configF = serverInfo.resolveServer(configFile);
        ApplicationContext factory = new FileSystemXmlApplicationContext(configF.toURL().toString());
        MutableServerStartupConfiguration cfg = (MutableServerStartupConfiguration) factory.getBean("configuration");
        Properties env = (Properties) factory.getBean("environment");
        env.setProperty(Context.PROVIDER_URL, providerURL);
        env.setProperty(Context.INITIAL_CONTEXT_FACTORY, ServerContextFactory.class.getName());

        File workingDirFile = serverInfo.resolveServer(workingDir);
        cfg.setWorkingDirectory(workingDirFile);

        env.putAll(cfg.toJndiEnvironment());
        new InitialDirContext(env);
    }


    public void doFail() {
        log.warn("Service failed");
        try {
            doStop();
        } catch (Exception e) {
            log.info("error during fail", e);
        }
    }

    public void doStart() throws Exception {
    }

    public void doStop() throws Exception {
        log.debug("Stopping LDAP Directory service");
        Properties env = new Properties();
        env.putAll(new ShutdownConfiguration().toJndiEnvironment());
        env.put(Context.INITIAL_CONTEXT_FACTORY, ServerContextFactory.class.getName());
        //The values of these env props are more or less irrelevant, see DIRSERVER-1002
        env.put(Context.SECURITY_AUTHENTICATION, "simple");
        env.put(Context.SECURITY_PRINCIPAL, "uid=admin,ou=system");
        env.put(Context.SECURITY_CREDENTIALS, "secret");

        ClassLoader oldCL = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(classLoader);

        try {
            new InitialDirContext(env);
        } finally {
            Thread.currentThread().setContextClassLoader(oldCL);
        }
    }

    public static final GBeanInfo GBEAN_INFO;

    static {
        GBeanInfoBuilder infoFactory = GBeanInfoBuilder.createStatic("Apache Directory LDAP", DirectoryGBean.class);

        infoFactory.addAttribute("workingDir", String.class, true);
        infoFactory.addAttribute("configFile", String.class, true);
        infoFactory.addAttribute("providerURL", String.class, true);
        infoFactory.addReference("ServerInfo", ServerInfo.class, "GBean");
        infoFactory.addAttribute("classLoader", ClassLoader.class, false);

        infoFactory.setConstructor(new String[]{"workingDir", "configFile", "providerURL", "ServerInfo", "classLoader"});
        GBEAN_INFO = infoFactory.getBeanInfo();
    }

    public static GBeanInfo getGBeanInfo() {
        return GBEAN_INFO;
    }

}
